#define ALLOW_ACCESS_TO_INTERNALS_OF_MENUS

#include <PalmOS.h>
#include "WbMnu.h"

typedef struct 
	{
	UInt16 On;
	UInt16 Off;
	UInt16 AutoEnbleF;
	} WbMnuPreferenceType;

#define appFileCreator			'WbMn'
#define appVersionNum			0x01
#define appPrefID				0x01
#define appPrefVersionNum		0x01

static void SetWbMnuPreference(WbMnuPreferenceType *prefs)
{
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		prefs, sizeof (WbMnuPreferenceType), false);
}

static void GetWbMnuPreference(WbMnuPreferenceType *prefs)
{
	UInt16 prefsSize;

	prefsSize = sizeof(WbMnuPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, prefs, &prefsSize, false) == 
		noPreferenceFound)
		{
			prefs->On         = 0;
			prefs->Off        = 1;
			prefs->AutoEnbleF = 0;

			SetWbMnuPreference(prefs);
		}
}

static void Disable(void)
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase (&cardNo, &dbID);
	SysNotifyUnregister(cardNo, dbID, sysNotifyEventDequeuedEvent, sysNotifyNormalPriority);
}

static void Enable(void)
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase (&cardNo, &dbID);
	SysNotifyRegister (cardNo, dbID, sysNotifyEventDequeuedEvent, NULL, sysNotifyNormalPriority, NULL);
}

static void GetCtrlValue(FormPtr frmP)
{
	WbMnuPreferenceType prefs;

	prefs.On         = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainONPushButton     ));
																		// !0:ON 0:OFF
	prefs.Off        = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainOFFPushButton    ));
																		// !0:ON 0:OFF
	prefs.AutoEnbleF = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainEnblRstCheckbox  ));
																		// !0:ON 0:OFF
	SetWbMnuPreference(&prefs);
	
	if (prefs.On != 0)
	{
		Enable();
	}
	else
	{
		Disable();
	}
}

static void SetCtrlValue(FormPtr frmP)
{
	WbMnuPreferenceType prefs;
	GetWbMnuPreference(&prefs);

	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainONPushButton     ), prefs.On        );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainOFFPushButton    ), prefs.Off       );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainEnblRstCheckbox  ), prefs.AutoEnbleF);
}

static UInt32 StarterPalmMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
//	Err error;

	switch (cmd)
		{
		case sysAppLaunchCmdNotify:
			SysNotifyParamType* NtfyPrm = (SysNotifyParamType*)cmdPBP;
			switch (NtfyPrm->notifyType)
				{
				case sysNotifyEventDequeuedEvent:
					EventPtr DequedEvent;
					DequedEvent = (EventType*)NtfyPrm->notifyDetailsP;
					if (DequedEvent->eType == 0x0400)		//keyDownEvent 0x0004
					{
						MenuBarType *menuP;
						menuP = MenuGetActiveMenu();
						if ( menuP != NULL && menuP->attr.visible)
						{
							EventType event;
								event.eType = (eventsEnum)0x0004;
								event.penDown = false;
								event.tapCount = 0;
								event.screenX = 0;
								event.screenY = 0;
								event.data.keyDown.chr = 0x1700;
								event.data.keyDown.keyCode = 0x0000;
								event.data.keyDown.modifiers = 0x0008;
							UInt16 times;
							UInt16 shownItemsCount = 0;

							switch (DequedEvent->data.keyDown.chr)
							{
							case 0x1e00:	//chrUpArrow 0x001E
								SysHandleEvent (&event);
								DequedEvent->eType = (eventsEnum)0x0000;
								break;

							case 0x1f00:	//chrDownArrow 0x001F
								event.data.keyDown.chr = 0x1701;
								SysHandleEvent (&event);
								DequedEvent->eType = (eventsEnum)0x0000;
								break;

							case 0x1c00:	//chrLeftArrow 0x001C
								UInt16 shownMenusCount = 0;
								if (menuP->numMenus != 0)
								{
									for (times = 0; times<menuP->numMenus; times++)
									{
										if (menuP->menus[times].hidden != true)
											shownMenusCount++;
									}
								}
								if (menuP->curItem != -1)
								{
									for (times = menuP->curItem; times > 0; times--)
									{
										if ( (menuP->menus[menuP->curMenu].items[times].hidden != true) &&
											(menuP->menus[menuP->curMenu].items[times].itemStr[0] != MenuSeparatorChar))
											shownItemsCount++;
									}
								}
								else
								{
									shownItemsCount = -1;
								}
								for (times = shownItemsCount + shownMenusCount;times > 0;times--)
									SysHandleEvent (&event);
								DequedEvent->eType = (eventsEnum)0x0000;
								break;

							case 0x1d00:	//chrRightArrow 0x001D
								if (menuP->curItem != -1)
								{
									for (times = menuP->curItem; times > 0; times--)
									{
										if ( (menuP->menus[menuP->curMenu].items[times].hidden != true) &&
											(menuP->menus[menuP->curMenu].items[times].itemStr[0] != MenuSeparatorChar))
											shownItemsCount++;
									}
								}
								else
								{
									shownItemsCount = -1;
								}
								for (times = shownItemsCount + 2;times > 0;times--)
									SysHandleEvent (&event);
								DequedEvent->eType = (eventsEnum)0x0000;
								break;

							case 0x0a00:	//chrLineFeed 0x000A
								DequedEvent->data.keyDown.chr = 0x0617;
														//vchrJogRelease 0x1706
								DequedEvent->data.keyDown.modifiers = 0x0800;
								break;

							case 0x0800:	//chrBackspace 0x0008
								DequedEvent->data.keyDown.chr = 0x0501;
								DequedEvent->data.keyDown.modifiers = 0x0800; 
									//vchrMenu 0x0105
								break;
							}
						}
					}
					break;
				}
			break;

		case sysAppLaunchCmdNormalLaunch:
			FormPtr frmP = FrmInitForm (MainForm);		//Initialize dialogue form
			SetCtrlValue(frmP);				 			//Set control value
			if (FrmDoDialog (frmP) == MainDoneButton)	//Draw dialogue form
				GetCtrlValue(frmP);						//Get control value
			break;

		case sysAppLaunchCmdSystemReset:
			WbMnuPreferenceType prefs;				//Check Auto Enable
			GetWbMnuPreference(&prefs);
			if (prefs.On != 0)
			{
				if (prefs.AutoEnbleF != 0)
					Enable();						//activate
				else
				{
					prefs.On  = 0;
					prefs.Off = 1;
					SetWbMnuPreference(&prefs);
				}
			}
			break;

		default:
			break;

		}
	
	return errNone;
}

UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	return StarterPalmMain(cmd, cmdPBP, launchFlags);
}
